﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later
#pragma IgorVersion=8			//Uses long names, so you need Igor 8 or later
#pragma version=1.0.0	


static constant kProjectID=22223
static strconstant ksShortTitle="FFmpeg tool"

//Code writen by:
//Kyle Zarzana
//University of Colorado Boulder

//Set of functions to call FFmpeg through Igor to make movies.  Requires that the computer knows where
//FFmpeg is installed. Works best when the image names are of the format [Image base name]_[Image number].
//The image number should always have the same number of digits.
//You CANNOT have any other files in the images folder except the images!!!  (I can't image why you would).


//Version history
//20200122: Added a button to only generate the batch file for Windows.  Added a control to change the crf value.  
//20200219: Renamed the ipf.
//
//20200514: Made the constants static, added a check to make sure the folder path ends in a colon
//20200819: Added checks to make sure that the images in the source folder match the entered Images Name format
//20211021: Replaced the function that creates the formatted paths with ParseFilePath(5).  I think it will now work on both Macs and PCs if there are spaces in any of the paths.  
//20211113: Added metadata when making a movie from a series of images.  Includes the epxeriment path, the path to the images folder, and the creation date.
//20211212: Added the ability to limit the number of threads used.  
//20220920: Changed control functions to use STRUCTS.  Renamed functions to better convention (Video_Tools_[Name]).
//20220921: When changing the speed of a video the original video metadata are now added to the metadata for the new video.  Changed the name of the source video controls on the
//			change speed tab to indicate that it's a file path, not a folder path.  
//20220927: Changed the encoder to H265, increased the default CRF to 28
//20220928: Added a popup to select the encoder.  Right now the only options are H264 and H265.  Removed the bitrate (for now).  I think it's redundant with CRF.  
//20230311: Added a setvar with the number of images in the folder
//20230613: Made sure all the control commands (ControlInfo and modifying values) reference the window panel

//v1.0.0	20230928: Removed the bitrate control.  Also took out the scaling control.  Added a tag so H265 videos work with QuickTime in macOS.  



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//You can also add FFmpeg to your system variables path, which removes the need to have the full path.  Using the full path seems a little more robust.
STATIC STRCONSTANT kstrFFmpeg_Path_Windows="C:ffmpeg:bin:ffmpeg"			//Use Igor formating here
STATIC STRCONSTANT kstrFFmpeg_Path_Mac="Macintosh HD:usr:local:bin:ffmpeg"		//Need the Macintosh HD at the start for ParseFilePath to work properly


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//I stuff a lot of tools into the Useful Tools menu.  
MENU "Useful Tools"

	SUBMENU "Video Tools"
		"Make panel", Video_Tools_Make_Panel()
	END
	
END


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


FUNCTION Video_Tools_Make_Panel()

	String strWin_Name=UniqueName("Video_Tools_Panel_", 9, 0)

	Variable vMax_Thread_Count=ThreadProcessorCount
	String strMax_Thread_Count=num2str(vMax_Thread_Count)

	NewPanel/W=(221,54,1410,290)/N=$strWin_Name as strWin_Name
	
	TabControl P0_Tab_Video_Panel,pos={2.00,6.00},size={1184.00,225.00},proc=Video_Tools_Tab_Controller,tabLabel(0)="Make Video",tabLabel(1)="Change Speed",value= 0
	
	//Tab 0 controls
	SetVariable T0_SetVar_Image_Source_Path,pos={10.00,35.00},size={1105.00,19.00},bodyWidth=1000,title="Images Folder Path",value= _STR:""
	Button T0_Button_Image_Source_Path,pos={1120,34.00},size={52.00,20.00},proc=Video_Tools_Button_Locate_Folder,title="Browse"
	
	SetVariable T0_SetVar_Image_Name,pos={38.00,60.00},size={407.00,19.00},bodyWidth=330,title="Images Name",value= _STR:""
	
	SetVariable T0_SetVar_Num_Images,pos={453.00,60.00},size={205.00,19.00},bodyWidth=100,title="Number of images:",format="%d",frame=0,limits={-inf,inf,0},value= _NUM:0,noedit= 1
	
	PopupMenu T0_Pop_Encoder,pos={117.00,85.00},size={95.00,19.00},title="Encoder",mode=2,popvalue="H265",value= #"\"H264;H265;\""
	SetVariable T0_SetVar_Framerate,pos={226.00,85.00},size={95.00,18.00},bodyWidth=35,title="Frame rate",value= _NUM:15
	
	CheckBox T0_Check_Set_CRF,pos={349.00,87.00},size={61.00,15.00},proc=Video_Tools_Check_CRF_Scale_Value,title="Set CRF?",value= 0
	SetVariable T0_SetVar_CRF_Value,pos={419.00,85.00},size={96.00,18.00},bodyWidth=40,title="CRF value",limits={0,inf,0},value= _NUM:28,disable=2
	TitleBox T0_Title_CRF,pos={519.00,87.00},size={276.00,15.00},title="(Lower is better, \"a subjectively sane range is 24–30\")",frame=0,disable=2		//https://trac.ffmpeg.org/wiki/Encode/H.264 and https://trac.ffmpeg.org/wiki/Encode/H.265

//	CheckBox T0_Check_Scale_Value,pos={828.00,87.00},size={48.00,15.00},title="Scale?",value= 0,proc=Video_Tools_Check_CRF_Scale_Value
//	SetVariable T0_SetVar_Scale_Value,pos={884.00,85.00},size={102.00,18.00},bodyWidth=40,title="Scale value",limits={0,inf,0},value= _NUM:800,disable=2
	
	SetVariable T0_SetVar_Video_Folder_Path,pos={17.00,110.00},size={1098.00,19.00},bodyWidth=1000,title="Video Folder Path",value= _STR:""
	Button T0_Button_Video_Folder_Path,pos={1120,109.00},size={52.00,20.00},proc=Video_Tools_Button_Locate_Folder,title="Browse"
	
	SetVariable T0_SetVar_File_Name,pos={58.00,135.00},size={357.00,19.00},bodyWidth=300,title="File Name",value= _STR:""
	
	Button T0_Button_Make_Video,pos={115.00,160.00},size={82.00,20.00},proc=Video_Tools_Button_Make_Video,title="Make Video",fStyle=1, fColor=(0,0,65535),valueColor=(65535,65535,65535), appearance={os9,all}
	CheckBox T0_Check_Erase_Bat,pos={202.00,162.00},size={184.00,16.00},title="Remove bat file? (Windows only)",value= 1
	
	SetVariable T0_SetVar_Num_Threads,pos={423.00,163.00},size={67.00,19.00},bodyWidth=40,title="Use ",value= _NUM:vMax_Thread_Count, limits={1,vMax_Thread_Count,1}
	TitleBox T0_Title_Thread_Count,pos={491.00,165.00},size={71.00,16.00},title="of "+strMax_Thread_Count+" threads",frame=0
	
	Button T0_Button_Make_Batch_File,pos={600,160.00},size={105,20.00},proc=Video_Tools_Button_Make_Video,title="Make Batch File",fColor=(65535,65535,65535)
	
	SetVariable T0_SetVar_Video_Command,pos={22.00,183.00},size={1093.00,19.00},bodyWidth=1000,title="Video Command",value= _STR:""
	
	//Tab 1 controls
	SetVariable T1_SetVar_Video_Source_File_Path,pos={10.00,35.00},size={1105.00,19.00},bodyWidth=1000,title="Source Video Path",value= _STR:"",disable=1
	Button T1_Button_Video_Source_File_Path,pos={1120,34.00},size={52.00,20.00},proc=Video_Tools_Button_Locate_File,title="Browse",disable=1
	
	SetVariable T1_SetVar_Dest_Folder_Path,pos={10.00,60.00},size={1105.00,19.00},bodyWidth=1000,title="Dest Folder Path",value= _STR:"",disable=1
	Button T1_Button_Dest_Folder_Path,pos={1120,59.00},size={52.00,20.00},proc=Video_Tools_Button_Locate_Folder,title="Browse",disable=1
	
	SetVariable T1_SetVar_File_Name,pos={58.00,85.00},size={367,19.00},bodyWidth=310,title="File Name",value= _STR:"",disable=1
	
	SetVariable T1_SetVar_Speed_Change,pos={58.00,110.00},size={97.00,19.00},bodyWidth=40,title="Speed change",value= _NUM:1,limits={-inf,inf,0},disable=1
	TitleBox T1_Title_Speed_Note,pos={164.00,112.00},size={509.00,16.00},title="Enter the factor you want to SLOW the video down by (e.g. 2 means the video will be twice as long)",frame=0,disable=1
	
	Button T1_Button_Change_Speed,pos={115.00,135.00},size={82.00,20.00},proc=Video_Tools_Button_Change_Speed,title="Make Video",disable=1,fStyle=1, fColor=(0,0,65535),valueColor=(65535,65535,65535), appearance={os9,all}
	CheckBox T1_Check_Erase_Bat,pos={202.00,137.00},size={184.00,16.00},disable=1,title="Remove bat file? (Windows only)",value= 1
	
	SetVariable T1_SetVar_Num_Threads,pos={445,136.00},size={67.00,19.00},bodyWidth=40,title="Use ",value= _NUM:vMax_Thread_Count,disable=1, limits={1,vMax_Thread_Count,1}
	TitleBox T1_Title_Thread_Count,pos={514,138},size={71.00,16.00},title="of "+strMax_Thread_Count+" threads",frame=0,disable=1
	
	SetVariable T1_SetVar_Video_Command,pos={22.00,159.00},size={1093.00,19.00},bodyWidth=1000,disable=1,title="Video Command",value= _STR:""
	
END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Enables/disables controls based on which tab is active
FUNCTION Video_Tools_Tab_Controller(tca) : TabControl
	STRUCT WMTabControlAction &tca

	SWITCH( tca.eventCode )
		CASE 2: // mouse up
	
			String strControls_2_Disable=ControlNameList("",";", "T*")											//Makes a list of all the controls on the tabs
			String strControls_2_Enable=ControlNameList("",";", ("T"+num2str(tca.tab)+"*"))					//Makes a list of the controls on the active tab
				
			ModifyControlList strControls_2_Disable, disable=1									//Disables all the controls on all the  tabs
			ModifyControlList strControls_2_Enable, disable=0									//Enables the controls on the active tabs
		
			IF(tca.tab==0)
				STRUCT WMCheckBoxAction cba
				
//				ControlInfo T0_Check_Scale_Value
//				cba.eventcode=2
//				cba.ctrlName="T0_Check_Scale_Value"
//				cba.checked=V_Value
//				Video_Tools_Check_CRF_Scale_Value(cba)
				
				ControlInfo T0_Check_Set_CRF
				cba.eventcode=2
				cba.ctrlName="T0_Check_Set_CRF"
				cba.checked=V_Value
				Video_Tools_Check_CRF_Scale_Value(cba)
			ENDIF
		BREAK
	ENDSWITCH

END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Enables/disables the scale set variable
FUNCTION Video_Tools_Check_CRF_Scale_Value(cba) : CheckBoxControl
	STRUCT WMCheckBoxAction &cba
	
	SWITCH(cba.eventcode)
		CASE 2:
		
			String strWin_Name=cba.win

			IF(StringMatch(cba.ctrlName, "T0_Check_Scale_Value"))
				SetVariable T0_SetVar_Scale_Value,win=$strWin_Name,disable=SelectNumber(cba.checked, 2, 0)
			ELSEIF(StringMatch(cba.ctrlName, "T0_Check_Set_CRF"))
				SetVariable T0_SetVar_CRF_Value,win=$strWin_Name,disable=SelectNumber(cba.checked, 2, 0)
				TitleBox T0_Title_CRF, win=$strWin_Name, disable=SelectNumber(cba.checked, 2, 0)
			ENDIF
		BREAK
	ENDSWITCH

END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Locates files
FUNCTION Video_Tools_Button_Locate_File(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SWITCH(ba.eventcode)
		CASE 2:
			
			String strWin_Name=ba.win
			
			Variable refNum
			String strMessage = "Select a file"
			
			String strSetVar_Ctrl_Name=ReplaceString("_Button_", ba.ctrlName, "_SetVar_")		//I can't imagine why _SetVar_ would be in the name twice, so this should work fine
				
			ControlInfo/W=$strWin_Name $strSetVar_Ctrl_Name
			IF(strlen(S_Value)==0)	//If the path is empty, then try the folder containing the experiment
				PathInfo/S home			
				
			ELSE		//Otherwise start in the folder containing the last folder that was selected
				String strFolder_Path=ParseFilePath(1, S_Value, ":", 1, 0)
				NewPath/O/Q/Z pPath, strFolder_Path
				PathInfo/S pPath
				KillPath/Z pPath
			ENDIF
			
			Open/D/R/T="????"/M=strMessage refNum			//The "????" means that Igor will not be looking for a particular file type
			
			IF(strlen(S_fileName)>0)
				SetVariable $strSetVar_Ctrl_Name, win=$strWin_Name, value=_STR:S_fileName
			ENDIF
			
			KillPath/Z pPath
		BREAK
	ENDSWITCH	

END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Function for finding folders
FUNCTION Video_Tools_Button_Locate_Folder(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SWITCH(ba.eventcode)
		CASE 2:
			
			String strWin_Name=ba.win
			
			String strSetVar_Ctrl_Name=ReplaceString("_Button_", ba.ctrlName, "_SetVar_")
			ControlInfo/W=$strWin_Name $strSetVar_Ctrl_Name
			
			IF(strlen(S_Value)==0)	//If the path is empty, then try the folder containing the experiment
				PathInfo/S home			
				
			ELSE		//Otherwise start in the folder containing the last folder that was selected
				IF(cmpstr(S_Value[strlen(S_Value)-1], ":")!=0)
					S_Value+=":"
				ENDIF
					
				String strFolder_Path=ParseFilePath(1, S_Value, ":", 1, 0)
				NewPath/O/Q/Z pPath, strFolder_Path
				PathInfo/S pPath
				KillPath/Z pPath
			ENDIF
		
			String strMessage="Select a folder"
			NewPath/Q/O/M=strMessage pTemp_Path			//Gets the path to a FOLDER (not a file)
			PathInfo pTemp_path
			
			IF(strlen(S_Path)>0)
				SetVariable $strSetVar_Ctrl_Name, win=$strWin_Name, value=_STR:S_Path			//Put the string in the setvariable with the same name as the button
				
				IF(cmpstr(ba.ctrlName, "T0_Button_Image_Source_Path")==0)			//If the button was for the path to the images, do some extra work to get the image names
					String strFolder_Name=ParseFilePath(0, S_Path, ":", 1, 0)
					
					WAVE/T All_Images=ListToTextWave(IndexedFile(pTemp_path, -1, "????"), ";")
					
					SetVariable T0_SetVar_Num_Images, win=$strWin_Name, value=_NUM:numpnts(All_Images)
					
					String strFile_Full_Name=IndexedFile(pTemp_path, 0, "????")
					IF(strlen(strFile_Full_Name)!=0)
						String strFile_Type=ParseFilePath(0, strFile_Full_Name, ".", 1, 0)		//Get the file extension (last item in a period separated path)
						String strFile=ParseFilePath(1, strFile_Full_Name, ".", 1, 0)				//Remove the file extension
						strFile=strFile[0,strlen(strFile)-2]				//Remove the period at the end	
						String strFile_Number=StringFromList(ItemsInList(strFile, "_")-1, strFile, "_")		//Grabs the last item in the underscore separated list
						
						String strFile_Name_Start=strFile[0,strlen(strFile)-2-strlen(strFile_Number)]
						
						String strFormat_String="%0"+num2str(strlen(strFile_Number))+"d"		//Gets the format string for the file number (Is there a better way to do this?)
						
						String strFile_Name_Template=strFile_Name_Start+"_"+strFormat_String+"."+strFile_Type
						
						SetVariable T0_SetVar_Image_Name, win=$strWin_Name, value=_STR:strFile_Name_Template
						SetVariable T0_SetVar_File_Name, win=$strWin_Name, value=_STR:strFolder_Name+".mp4"
						
					ELSE
						SetVariable T0_SetVar_Image_Name, win=$strWin_Name, value=_STR:"NO IMAGES IN FOLDER!!!"
						
						DoAlert/T="ERROR ID-10T" 0, "Selected folder contains no images!"
					ENDIF
				ENDIF
			ENDIF
			
			KillPath/Z pTemp_path
		BREAK
	ENDSWITCH
	
END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Creates the command to make the video and executes a batch file with that command (Windows) or runs the Applescript (Mac).
//Also gets called by the button that just makes the batch file on Windows.  
FUNCTION Video_Tools_Button_Make_Video(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SWITCH(ba.eventcode)
		CASE 2:
	
			String strWin_Name=ba.win
			
			Variable vIs_Windows=StringMatch(IgorInfo(2), "*Win*")			//Checks the OS
			String strPath_Separator=SelectString(vIs_Windows, "/", "\\")
			
			PathInfo home
			String strPXP_Path=S_Path
		
			String strPXP_Name=IgorInfo(1)
			String strDate=Secs2Date(DateTime, -2)
			String strTime=Secs2Time(DateTime, 3)
			String strUTC_Offset=num2str(floor(Date2Secs(-1,-1,-1)/3600))
		
			String strInfo_String="Experiment="+strPXP_Path+strPXP_Name+".pxp;Save date="+strDate+" "+strTime+" (UTC"+strUTC_Offset+");"
			
			String strFFmpeg_path_format=ParseFilePath(5, SelectString(vIs_Windows, kstrFFmpeg_Path_Mac, kstrFFmpeg_Path_Windows), strPath_Separator, 0, 0)		//Creates the ffmpeg path
			
			ControlInfo/W=$strWin_Name T0_SetVar_Video_Folder_Path
			String strVideo_Dest_Folder_Path=S_Value
			IF(strlen(strVideo_Dest_Folder_Path)==0)
				DoAlert/T="WARNING" 0, "Select a folder for the video!"
				Return -1
			ENDIF
			
			GetFileFolderInfo/Q/Z strVideo_Dest_Folder_Path
			IF(V_Flag!=0)
				DoAlert/T="ERROR ID-10T!!" 0, "Video destination folder does not exist!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			
			//Get the image source folder
			ControlInfo/W=$strWin_Name T0_SetVar_Image_Source_Path
			String strImage_Source_Path=S_Value
			GetFileFolderInfo/Q/Z strImage_Source_Path
			IF(V_Flag!=0)
				DoAlert/T="ERROR ID-10T!!" 0, "Image source folder does not exist!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			
			strInfo_String+="Frame folder path="+strImage_Source_Path+";"
			
		
			//Get the image name	
			ControlInfo/W=$strWin_Name T0_SetVar_Image_Name
			String strImage_Name=S_Value
			String strImage_Name_Format=SelectString(vIs_Windows, strImage_Name, ReplaceString("%", strImage_Name, "%%%%"))		//Batch file needs double % to run, and need double % to print a single %.
			
			//Make sure there are valid images
			String strImage_Type=ParseFilePath(0, strImage_Name, ".", 1, 0)
			NewPath/O/Q/Z pImage_Source_Path, strImage_Source_Path
			String strImage_Files=IndexedFile(pImage_Source_Path, -1, "."+strImage_Type)			
			IF(strlen(strImage_Files)==0)
				DoAlert/T="ERROR ID-10T!!" 0, "Image source folder is empty!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			
			KillPath/Z pImage_Source_Path
			
			Variable vLast_Underscore=strsearch(strImage_Name, "_", strlen(strImage_Name)-1, 3)
			IF(vLast_Underscore<0)
				DoAlert/T="ERROR ID-10T!!" 0, "Probably an invalid name format."
				Return -1
			ENDIF
			
			String strMatch_Str=strImage_Name[0,vLast_Underscore-1]+"*."+strImage_Type
			
			WAVE/T twImage_Files=ListToTextWave(strImage_Files, ";")
			Extract/O/T/FREE twImage_Files, twImage_Files_Valid, StringMatch(twImage_Files, strMatch_Str)
			IF(numpnts(twImage_Files_Valid)==0)
				DoAlert/T="ERROR ID-10T!!" 0, "Image source folder contains no files with the correct name format!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			
			String strImage_Source_Path_Format=ParseFilePath(5, strImage_Source_Path, strPath_Separator , 0, 0)
			String strImage_Source_Path_Name_Format=strImage_Source_Path_Format+strImage_Name_Format
			
			ControlInfo/W=$strWin_Name T0_SetVar_Framerate
			String strFramerate=num2str(V_Value)
			
			String strEncoder, strTag
			ControlInfo/W=$strWin_Name T0_Pop_Encoder
			STRSWITCH(S_Value)
				CASE "H264":
					strEncoder="libx264"
					strTag=""
				BREAK
				
				CASE "H265":
					strEncoder="libx265"
					strTag="-tag:v hvc1 "
				BREAK
			ENDSWITCH
			
			String strCRF_Value=""
			ControlInfo/W=$strWin_Name T0_Check_Set_CRF
			IF(V_Value)
				ControlInfo/W=$strWin_Name T0_SetVar_CRF_Value
				strCRF_Value="-crf "+num2str(V_Value)+" "
			ENDIF
			
//			String strScale_Value=""
//			ControlInfo/W=$strWin_Name T0_Check_Scale_Value
//			IF(V_Value)
//				ControlInfo/W=$strWin_Name T0_SetVar_Scale_Value
//				strScale_Value="-vf scale="+num2str(V_Value)+":-1 "
//			ENDIF
			
			ControlInfo/W=$strWin_Name T0_SetVar_File_Name
			String strVideo_File_Name=S_Value
			String strVideo_File_Name_No_Extension=StringFromList(0, strVideo_File_Name, ".")
			
			Variable vTime=DateTime
			String strDateTime=ReplaceString("-", Secs2Date(vTime, -2), "")+"_"+ReplaceString(":", Secs2Time(vTime, 3), "")
			
			String strVideo_Dest_Folder_Path_Format=ParseFilePath(5, strVideo_Dest_Folder_Path, strPath_Separator, 0, 0)	
			String strVideo_Dest_File_Path_Format=strVideo_Dest_Folder_Path_Format+strVideo_File_Name
			
			GetFileFolderInfo/Q/Z strVideo_Dest_Folder_Path+strVideo_File_Name
			IF(V_IsFile)
				DoAlert/T="WARNING" 0, "Video file already exists!  Deal with this before running FFmpeg!"
				Return -1
			ENDIF
			
			Variable vMax_Thread_Count=ThreadProcessorCount
			ControlInfo/W=$strWin_Name T0_SetVar_Num_Threads
			Variable vNum_User_Threads=min(V_Value, vMax_Thread_Count)
			
			String strThread_Command=""
			IF(vMax_Thread_Count!=vNum_User_Threads)
				strThread_Command="-threads "+num2str(vNum_User_Threads)+" "
			ENDIF
						
//			String strTerminal_Command=strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Format+strImage_Name+" -c:v "+strEncoder+" -b:v "+strBit_Rate+" -pix_fmt yuv420p -vf scale=dst_range=1 "+strCRF_Value+strThread_Command+strVideo_Dest_File_Path_Format
			String strTerminal_Command=strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Format+strImage_Name+" -c:v "+strEncoder+" -pix_fmt yuv420p -vf scale=dst_range=1 "+strTag+strCRF_Value+strThread_Command+strVideo_Dest_File_Path_Format
			SetVariable T0_SetVar_Video_Command, win=$strWin_Name, value=_STR:strTerminal_Command
			
			IF(vIs_Windows)														//use full paths for everything so I don't have to use cd
				strImage_Source_Path_Name_Format="\""+strImage_Source_Path_Name_Format+"\""
				
				IF(StringMatch(strVideo_Dest_File_Path_Format, "* *"))
					strVideo_Dest_File_Path_Format="\""+strVideo_Dest_File_Path_Format+"\""
				ENDIF
				
				strInfo_String="\""+strInfo_String+"\""
				
//				String strBatch_Command=strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Name_Format+" -c:v "+strEncoder+" -b:v "+strBit_Rate+" -pix_fmt yuv420p -vf scale=dst_range=1 "+strCRF_Value+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format
				String strBatch_Command=strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Name_Format+" -c:v "+strEncoder+" -pix_fmt yuv420p -vf scale=dst_range=1 "+strTag+strCRF_Value+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format
		//		Print strBatch_Command
						
				String strBatch_Path=strVideo_Dest_Folder_Path+strVideo_File_Name_No_Extension+"_"+strDateTime+"_Batch.bat"
				String strBatch_Path_Format=ParseFilePath(5, strBatch_Path, strPath_Separator, 0, 0)
							
				Variable vFileID_Batch	
				Open vFileID_Batch as strBatch_Path		
				fprintf vFileID_Batch, strBatch_Command					//Write the new string to the batch file
				Close vFileID_Batch
				
				IF(!StringMatch(ba.ctrlName, "T0_Button_Make_Batch_File"))
					IF(StringMatch(strBatch_Path_Format, "* *"))
						strBatch_Path_Format="\""+strBatch_Path_Format+"\""
					ENDIF
					
					ExecuteScriptText strBatch_Path_Format
							
					ControlInfo/W=$strWin_Name T0_Check_Erase_Bat		//Kill the batch file if the user wants
					IF(V_Value)
						DeleteFile/Z strBatch_Path
					ENDIF
				ENDIF
				
			ELSE		//Mac code
				IF(!StringMatch(ba.ctrlName, "T0_Button_Make_Batch_File"))
					IF(StringMatch(strVideo_Dest_File_Path_Format, "* *"))
						strVideo_Dest_File_Path_Format="'"+strVideo_Dest_File_Path_Format+"'"
					ENDIF
					
					IF(StringMatch(strImage_Source_Path_Name_Format, "* *"))
						strImage_Source_Path_Name_Format="'"+strImage_Source_Path_Name_Format+"'"
					ENDIF
					
					strInfo_String="'"+strInfo_String+"'"
				
//					String Script_Command="do shell script \""+strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Name_Format+" -c:v "+strEncoder+" -b:v "+strBit_Rate+" -pix_fmt yuv420p "+strScale_Value+strCRF_Value+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format+"\""
					String Script_Command="do shell script \""+strFFmpeg_path_format+" -framerate "+strFramerate+" -i "+strImage_Source_Path_Name_Format+" -c:v "+strEncoder+" -pix_fmt yuv420p -vf scale=dst_range=1 "+strTag+strCRF_Value+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format+"\""
					ExecuteScriptText Script_Command
				ENDIF
			ENDIF
		BREAK
	ENDSWITCH
	
END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///Creates the command to change the video speed and executes a batch file with that command (Windows) or runs the Applescript (Mac)
FUNCTION Video_Tools_Button_Change_Speed(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SWITCH(ba.eventcode)
		CASE 2:
	
			String strWin_Name=ba.win
			
			Variable vIs_Windows=StringMatch(IgorInfo(2), "*Win*")			//Checks the OS
			String strPath_Separator=SelectString(vIs_Windows, "/", "\\")
			
			PathInfo home
			String strPXP_Path=S_Path
		
			String strPXP_Name=IgorInfo(1)
			String strDate=Secs2Date(DateTime, -2)
			String strTime=Secs2Time(DateTime, 3)
			String strUTC_Offset=num2str(floor(Date2Secs(-1,-1,-1)/3600))
		
			//Need to read the old info string, and then add the speed info to that
			//start here: ffmpeg -i in.mp4 -f ffmetadata in.txt
			
			String strInfo_String="Experiment="+strPXP_Path+strPXP_Name+".pxp;Save date="+strDate+" "+strTime+" (UTC"+strUTC_Offset+");"
			
			String strFFmpeg_path_format=ParseFilePath(5, SelectString(vIs_Windows, kstrFFmpeg_Path_Mac, kstrFFmpeg_Path_Windows), strPath_Separator, 0, 0)
			
			ControlInfo/W=$strWin_Name T1_SetVar_Video_Source_File_Path
			String strSource_File_Path=S_Value
			GetFileFolderInfo/Q/Z strSource_File_Path
			IF(V_Flag!=0)
				DoAlert/T="ERROR ID-10T!!" 0, "Source file does not exist!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			String strSource_File_Path_Format=ParseFilePath(5, strSource_File_Path, strPath_Separator, 0, 0)
			String strSource_Extension=StringFromList(1, ParseFilePath(0, strSource_File_Path, ":", 1, 0), ".")
			strInfo_String+="Source file path="+strSource_File_Path+";"
									
			ControlInfo/W=$strWin_Name T1_SetVar_File_Name
			String strFile_Name=S_Value
			String strFile_Name_No_Extension=StringFromList(0, strFile_Name, ".")
			IF(strlen(StringFromList(1, strFile_Name, "."))==0)			//Make sure the user added an extension to the new file name
				strFile_Name=strFile_Name+"."+strSource_Extension
				SetVariable T1_SetVar_File_Name, win=$strWin_Name, value=_STR:strFile_Name
			ENDIF
			
			ControlInfo/W=$strWin_Name T1_SetVar_Dest_Folder_Path
			String strDest_Folder_Path=S_Value
			GetFileFolderInfo/Q/Z strDest_Folder_Path
			IF(V_Flag!=0)
				DoAlert/T="ERROR ID-10T!!" 0, "Destination folder does not exist!  I'm really not sure what you are expecting to happen here..."
				Return -1
			ENDIF
			String strVideo_Dest_File_Path_Format=ParseFilePath(5, strDest_Folder_Path, strPath_Separator, 0, 0)+strFile_Name
			
			ControlInfo/W=$strWin_Name T1_SetVar_Speed_Change
			Variable vPTS=V_Value
			String strPTS
			sprintf strPTS, "%0.04f", vPTS
			strInfo_String+="Speed reduction factor="+strPTS+";"
						
			Variable vTime=DateTime
			String strDateTime=ReplaceString("-", Secs2Date(vTime, -2), "")+"_"+ReplaceString(":", Secs2Time(vTime, 3), "")
			
			GetFileFolderInfo/Q/Z strDest_Folder_Path+strFile_Name
			IF(V_IsFile)
				DoAlert/T="WARNING" 0, "Video file already exists!  Deal with this before running FFmpeg!"
				Return -1
			ENDIF
			
			Variable vMax_Thread_Count=ThreadProcessorCount
			ControlInfo/W=$strWin_Name T0_SetVar_Num_Threads
			Variable vNum_User_Threads=V_Value
			
			String strThread_Command=""
			IF(vMax_Thread_Count!=vNum_User_Threads)
				strThread_Command="-threads "+num2str(vNum_User_Threads)+" "
			ENDIF
			
			ControlInfo/W=$strWin_Name T0_Check_Erase_Bat		//Kill the batch file if the user wants
			Variable vKill_Files=V_Value
		
			String strOriginal_Meta_Data=""
		
			IF(vIs_Windows)
				IF(StringMatch(strSource_File_Path_Format, "* *"))
					strSource_File_Path_Format="\""+strSource_File_Path_Format+"\""
				ENDIF
				
				IF(StringMatch(strVideo_Dest_File_Path_Format, "* *"))
					strVideo_Dest_File_Path_Format="\""+strVideo_Dest_File_Path_Format+"\""
				ENDIF
				
				String strMeta_Data_File_Path=strDest_Folder_Path+strFile_Name_No_Extension+"_"+strDateTime+"_Metadata.txt"
				String strMeta_Data_File_Path_Format=ParseFilePath(5, strMeta_Data_File_Path, strPath_Separator, 0, 0)
				IF(StringMatch(strMeta_Data_File_Path_Format, "* *"))
					strMeta_Data_File_Path_Format="\""+strMeta_Data_File_Path_Format+"\""
				ENDIF
				
				String strMeta_Data_Batch_File_Path=strDest_Folder_Path+strFile_Name_No_Extension+"_"+strDateTime+"_Metadata_Batch.bat"
				String strMeta_Data_Batch_File_Path_Format=ParseFilePath(5, strMeta_Data_Batch_File_Path, strPath_Separator, 0, 0)
				IF(StringMatch(strMeta_Data_Batch_File_Path_Format, "* *"))
					strMeta_Data_Batch_File_Path_Format="\""+strMeta_Data_Batch_File_Path_Format+"\""
				ENDIF
				
				String strMeta_Data_Batch_Command=strFFmpeg_path_format+" -i "+strSource_File_Path_Format+" -f ffmetadata "+strMeta_Data_File_Path_Format
				Variable vFileID_Meta
				Open vFileID_Meta as strMeta_Data_Batch_File_Path		
				fprintf vFileID_Meta, strMeta_Data_Batch_Command					//Write the new string to the batch file
				Close vFileID_Meta
				
				ExecuteScriptText strMeta_Data_Batch_File_Path_Format
				
				GetFileFolderInfo/Q/Z strMeta_Data_File_Path
				IF(V_Flag==0 && V_IsFile)
					String strCurrent_Data_Folder=GetDataFolder(1)
					NewDataFolder/O/S root:Temporary_FFmpeg_Metadata_Folder
					LoadWave/J/D/K=2/V={""," $",0,0}/O/A/B="N=twMeta_Data;"/Q strMeta_Data_File_Path
					
					WAVE/T/Z twMeta_Data=root:Temporary_FFmpeg_Metadata_Folder:twMeta_Data
					FindValue/TEXT="comment=" twMeta_Data
					IF(V_Value>=0)
						strOriginal_Meta_Data=(twMeta_Data[V_Value])[8,strlen(twMeta_Data[V_Value])]
						strOriginal_Meta_Data="Original video file info:"+ReplaceString("\\", strOriginal_Meta_Data, "")
					ENDIF
					
					SetDataFolder $strCurrent_Data_Folder
					KillDataFolder/Z root:Temporary_FFmpeg_Metadata_Folder
				ENDIF
				
				IF(vKill_Files)
					DeleteFile/Z strMeta_Data_Batch_File_Path
					DeleteFile/Z strMeta_Data_File_Path
				ENDIF
				
				strInfo_String+=strOriginal_Meta_Data
				strInfo_String="\""+strInfo_String+"\""
				
				String strBatch_File_Path=strDest_Folder_Path+strFile_Name_No_Extension+"_"+strDateTime+"_Batch.bat"
				String strBatch_File_Path_Format=ParseFilePath(5, strBatch_File_Path, strPath_Separator, 0, 0)
				
				IF(StringMatch(strBatch_File_Path_Format, "* *"))
					strBatch_File_Path_Format="\""+strBatch_File_Path_Format+"\""
				ENDIF
				
				String strBatch_Command=strFFmpeg_path_format+" -i "+strSource_File_Path_Format+" -filter:v \"setpts="+strPTS+"*PTS\" "+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format
				
				Variable vFileID_Batch	
				Open vFileID_Batch as strBatch_File_Path		
				fprintf vFileID_Batch, strBatch_Command					//Write the new string to the batch file
				Close vFileID_Batch
				
				SetVariable T1_SetVar_Video_Command, win=$strWin_Name, value=_STR:strBatch_Command
				
				ExecuteScriptText strBatch_File_Path_Format
						
				IF(vKill_Files)
					DeleteFile/Z strBatch_File_Path
				ENDIF
				
			ELSE		//Mac code
				IF(StringMatch(strSource_File_Path_Format, "* *"))
					strSource_File_Path_Format="'"+strSource_File_Path_Format+"'"
				ENDIF
				
				IF(StringMatch(strVideo_Dest_File_Path_Format, "* *"))
					strVideo_Dest_File_Path_Format="'"+strVideo_Dest_File_Path_Format+"'"
				ENDIF
				
				strMeta_Data_File_Path=strDest_Folder_Path+strFile_Name_No_Extension+"_"+strDateTime+"_Metadata.txt"
				strMeta_Data_File_Path_Format=ParseFilePath(5, strDest_Folder_Path, strPath_Separator, 0, 0)+strFile_Name_No_Extension+"_"+strDateTime+"_Metadata.txt"				
				IF(StringMatch(strMeta_Data_File_Path_Format, "* *"))
					strMeta_Data_File_Path_Format="'"+strMeta_Data_File_Path_Format+"'"
				ENDIF
				
				String strMeta_Data_Script_Command="do shell script \""+strFFmpeg_path_format+" -i "+strSource_File_Path_Format+" -f ffmetadata "+strMeta_Data_File_Path_Format+"\""
				ExecuteScriptText strMeta_Data_Script_Command	
				
				GetFileFolderInfo/Q/Z strMeta_Data_File_Path
				IF(V_Flag==0 && V_IsFile)
					strCurrent_Data_Folder=GetDataFolder(1)
					NewDataFolder/O/S root:Temporary_FFmpeg_Metadata_Folder
					LoadWave/J/D/K=2/V={""," $",0,0}/O/A/B="N=twMeta_Data;"/Q strMeta_Data_File_Path
					
					WAVE/T/Z twMeta_Data=root:Temporary_FFmpeg_Metadata_Folder:twMeta_Data
					FindValue/TEXT="comment=" twMeta_Data
					IF(V_Value>=0)
						strOriginal_Meta_Data=(twMeta_Data[V_Value])[8,strlen(twMeta_Data[V_Value])]
						strOriginal_Meta_Data="Original video file info:"+ReplaceString("\\", strOriginal_Meta_Data, "")
					ENDIF
					
					SetDataFolder $strCurrent_Data_Folder
					KillDataFolder/Z root:Temporary_FFmpeg_Metadata_Folder
				ENDIF
				
				IF(vKill_Files)
					DeleteFile/Z strMeta_Data_File_Path
				ENDIF
				
				strInfo_String+=strOriginal_Meta_Data
				strInfo_String="'"+strInfo_String+"'"
							
				String Script_Command="do shell script \""+strFFmpeg_path_format+" -i "+strSource_File_Path_Format+" -filter:v setpts="+strPTS+"*PTS "+strThread_Command+" -metadata comment="+strInfo_String+" "+strVideo_Dest_File_Path_Format+"\""
				ExecuteScriptText Script_Command	
			ENDIF
		BREAK
	ENDSWITCH
	
END


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

